#ifndef __MVDMA_IOCTL_H__
#define __MVDMA_IOCTL_H__

/*
 *  mvPCIe Driver - DMA implementation
 *  Copyright (c) 2019-2022 by MATRIX VISION GmbH <info@matrix-vision.de>
 *
 */

#ifdef linux

#include <linux/types.h>

typedef struct mvDMA_UserBuffer {
    void *buffer;
    size_t size;
    int32_t timeout;
    uint64_t dummy1;            /* deprecated, soft timestamp no longer used */
    uint64_t dummy2;            /* deprecated, soft timestamp no longer used */
    uint32_t img_size;
    uint32_t img_id;
} __attribute__ ((__packed__)) TmvDMA_UserBuffer;

#define MVDMA_IOC_QUEUE_BUFFER _IOWR(0, 2, TmvDMA_UserBuffer)
#define MVDMA_IOC_DEQUEUE_BUFFER _IOWR(0, 3, TmvDMA_UserBuffer)
#define MVDMA_IOC_ABORT _IOW(0, 4, uint64_t)
#define MVDMA_IOC_DMA_ENABLE _IOW(0, 5, uint32_t)
#define MVDMA_IOC_RESET_DMA _IOW(0, 6, uint32_t)

#define MVDMA_QUEUE_CNT 1024    /* Do not change: this is also fixed in the FPGA */
#define MVDMA_PAGE_LIST_CNT 64

#define MVPCI_IOW_SEQUENCER_SIZE    (256 * 1024LL)  /* size of memory used by io_writer for the sequencer control */
#define MVPCI_IOW_SEQUENCER_OFFSET  (4 * 1024LL * 1024LL)   /* offset to be used for file access to sequencer memory from user space */

typedef struct mvPCI_Register32 {
    uint32_t offset;
    uint32_t value;
    size_t size;
} __attribute__ ((__packed__)) TmvPCI_Register32;

typedef struct mvPCI_user_info {
    uint32_t streaming_owner;
    uint32_t user_count;
} __attribute__ ((__packed__)) TmvPCI_user_info;

#define MVPCI_IOC_READ_BAR  _IOR(1, 0, TmvPCI_Register32)
#define MVPCI_IOC_WRITE_BAR _IOW(1, 1, TmvPCI_Register32)
#define MVPCI_IOC_READ_MAXPAYLOAD _IOR(1, 2, uint32_t)
#define MVPCI_IOC_READ_MAXREADREQ _IOR(1, 3, uint32_t)
#define MVPCI_IOC_READ_FPGA_FAMILY _IOR(1, 4, uint32_t)
#define MVPCI_IOC_READ_FPGA_VERSION _IOR(1, 5, uint32_t)
#define MVPCI_IOC_READ_IOWSEQ_BASE_ADDR _IOR(1, 6, uint64_t)
#define MVPCI_IOC_READ_PCIE_BANDWIDTH _IOR(1, 7, uint32_t)
#define MVPCI_IOC_READ_PCIE_LINK_WIDTH _IOR(1, 8, uint32_t)
#define MVPCI_IOC_READ_PCIE_BUS_SPEED _IOR(1, 9, uint32_t)
#define MVPCI_IOC_WRITE_MAXPAYLOAD _IOW(1, 10, uint32_t)
#define MVPCI_IOC_READ_USER_INFO _IOR(1, 11, TmvPCI_user_info)
#define MVPCI_IOC_WRITE_USER_INFO _IOW(1, 12, TmvPCI_user_info)
#define MVPCI_IOC_READ_PCIE_DEVID _IOR(1, 13, uint32_t)

#endif /* __MVDMA_IOCTL_H__ */

#endif /* linux */
